from ArcUUID.ArcUUID import ArcUUID


class Ids:

    WAPO = ArcUUID.from_string("PW5TIZPZINHXBPNQTAJRSISO3E")
    DEMO1 = ArcUUID.from_string("2HDHHWH45JAHDFRWJKX2TJDVFA")
    DEMO2 = ArcUUID.from_string("P36DKL6XL5HKFEK33TSRHKDXRM")
    DEMO3 = ArcUUID.from_string("QFJBYOYVDZC35N2X33UBMMCUNI")

    def __init__(self):
        pass




import uuid
import base64
import binascii

class ArcUUID:

    LENGTH = 26

    def __init__(self, str_arc_uuid=None):
        if not str_arc_uuid:
            gen_uuid = uuid.uuid4()
            self.id = str(base64.b32encode(gen_uuid.bytes)).replace('=', '').replace('b\'', '').replace('\'', '') # to remove padding
        else:
            if len(str_arc_uuid) != ArcUUID.LENGTH:
                raise ValueError("This arcUUID doesn't meet the length expectations of an ArcUUID:" + str_arc_uuid)
            try:
                # just to see if string has non-alphabet characters. we already know length is 26 so we pad by 6 ='s to
                # length 32 to meet base32 spec
                base64.b32decode(str_arc_uuid + "======")
            except binascii.Error:
                raise ValueError("This arcUUID has characters that aren't in the Base32 encoding:" + str_arc_uuid)
            self.id = str_arc_uuid

    def __str__(self):
        return self.id

    @staticmethod
    def random_arc_uuid():
        return ArcUUID()

    @staticmethod
    def from_string(str_arc_uuid):
        if not str_arc_uuid:
            raise ValueError("Cannot construct an ArcUUID off a None value!")
        return ArcUUID(str_arc_uuid=str_arc_uuid)

    def __hash__(self):
        hash_code = 5
        hash_code = 43 * hash_code + self.id.__hash__()
        return hash_code

    def __eq__(self, other):
        if not other:
            return False
        if not isinstance(other, ArcUUID):
            return False
        if not self.id == other.id:
            return False
        return True



import base64
import unittest
import uuid

from ArcUUID.ArcUUID import ArcUUID
from ArcUUID.Ids import Ids


class ArcUUIDTest(unittest.TestCase):

    def test_unique_id_max(self):
        uuid_max = uuid.UUID('{7fffffff-ffff-ffff-7fff-ffffffffffff}')
        max_encoded = base64.b32encode(uuid_max.bytes)
        max_encoded = str(max_encoded).replace('=', '').replace('b\'', '').replace('\'', '')
        self.assertEqual("P7777777777767777777777774", max_encoded)
        self.assertEqual(ArcUUID.LENGTH, len(max_encoded))

    def test_unique_id_min(self):
        uuid_min = uuid.UUID('{80000000-0000-0000-8000-000000000000}')
        min_encoded = base64.b32encode(uuid_min.bytes)
        min_encoded = str(min_encoded).replace('=', '').replace('b\'', '').replace('\'', '')
        self.assertEqual("QAAAAAAAAAAABAAAAAAAAAAAAA", min_encoded)
        self.assertEqual(ArcUUID.LENGTH, len(min_encoded))
        self.assertIsNotNone(ArcUUID.from_string("QAAAAAAAAAAABAAAAAAAAAAAAA"))

    def test_unique_id(self):
        unique_id = str(ArcUUID.random_arc_uuid());
        self.assertEqual(len(unique_id), ArcUUID.LENGTH, "uuid length should be 26");

    def test_hardcoded_washington_post_uuid(self):
        wapoID = "PW5TIZPZINHXBPNQTAJRSISO3E"
        self.assertEqual(26, len(wapoID))
        wapoUUID = ArcUUID.from_string(wapoID)
        self.assertEqual(wapoUUID, Ids.WAPO)

    def test_arc_uuid_from_null_string_throws_exception(self):
        with self.assertRaises(ValueError):
            ArcUUID.from_string(None)

    def test_arc_uuid_with_too_long_string_throws_exception(self):
        with self.assertRaises(ValueError):
            ArcUUID.from_string("QAAAAAAAAAAABAAAAAAAAAAAAAQAAAAAAAAAAABAAAAAAAAAAAAA");

    def test_arc_uuid_with_not_base32_characters_throws_exception(self):
        with self.assertRaises(ValueError):
            ArcUUID.from_string("!@#$%^&*()================");

    def test_equals_contract(self):
        check1 = Ids.DEMO1
        check2 = ArcUUID.from_string("2HDHHWH45JAHDFRWJKX2TJDVFA")
        self.assertEqual(check1, check2)
        check3 = Ids.DEMO2
        check4 = ArcUUID.from_string("QFJBYOYVDZC35N2X33UBMMCUNI")
        self.assertNotEqual(check3, check4)



from setuptools import setup
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='py-arc-identifiers',
    version='0.1',
    description='generate Arc UUIDs in python',
    long_description=long_description,
    url='https://github.com/MPMedia/py-arc-identifiers',
    download_url='https://github.com/WPMedia/py-arc-identifiers/tarball/0.2.4',
    author='patharer',
    author_email='rohan.pathare@washpost.com',
    license='MIT',
    packages=['ArcUUID', 'test'],
)